---
title: Guides Index
sections:
  - name: Schema
  - name: Queries
  - name: Type Definitions
  - name: Authorization
  - name: Fields
  - name: Mutations
  - name: Errors
  - name: Pagination
  - name: Relay
  - name: Dataloader
  - name: Subscriptions
  - name: GraphQL Pro
  - name: GraphQL Pro - OperationStore
  - name: GraphQL Pro - Defer
  - name: GraphQL Enterprise - Rate Limiters
  - name: GraphQL Enterprise - Object Cache
  - name: GraphQL Enterprise - Changesets
  - name: JavaScript Client
  - name: Language Tools
  - name: Testing
  - name: Other
---

<h1>Guides</h1>
<div class="guides-toc">
  <ul>
    {% for section in page.sections %}
      <li>
        <a href="#{{ section.name | slugify }}-guides">{{ section.name }}</a>
      </li>
    {% endfor %}
  </ul>
</div>
<div class="guides">
  {% assign sorted_pages = site.pages | sort: "index" %}
  {% assign pages_with_index = ''|split:'' %}
  {% assign pages_without_index = ''|split:'' %}
  {% for guide in sorted_pages %}
    {% if guide.layout == "guide" %}
      {% if guide.index %}
        {% assign pages_with_index = pages_with_index | push: guide %}
      {% else %}
        {% assign pages_without_index = pages_without_index | push: guide %}
      {% endif %}
    {% endif %}
  {% endfor %}

  {% assign sorted_guides = pages_with_index | concat: pages_without_index %}
  {% for section in page.sections %}
    <div class="guide-section">
      <h2 id="{{ section.name | slugify }}-guides">{{ section.name }}</h2>
      <ul>
        {% for guide in sorted_guides %}
          {% if guide.section == section.name %}
            <li>
              <p>
                <a href="{% if guide.outbound_url %}{{ guide.outbound_url }}{% else %}{{ site.baseurl }}{{ guide.url }}{% endif %}">{{ guide.title }}</a>
                <span class="guide-desc">
                  {{ guide.desc }}
                </span>
              </p>
            </li>
          {% endif %}
        {% endfor %}
      </ul>
    </div>
  {% endfor %}
</div>
