<!DOCTYPE HTML>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    {% if page.section contains "GraphQL" %}
      <title>{{ page.section }} - {{ page.title }}</title>
    {% else %}
      <title>GraphQL - {{ page.title }}</title>
    {% endif %}
    <link href="https://fonts.googleapis.com/css?family=Rubik:300,400,700" rel="stylesheet" />
    <link rel="stylesheet" href="{{ site.baseurl }}/css/main.css">
    <link rel="icon" href="{{ site.baseurl }}/graphql-ruby-icon.png">
  </head>
  <body>
    <div class="header">
      <div class="header-container">
        <div class="nav">
          <a href="{{ site.baseurl }}/" class="img-link">
            <img src="{{ site.baseurl }}/graphql-ruby.png" alt="GraphQL Ruby Logo" />
          </a>
          <a href="{{ site.baseurl }}/getting_started">Get Started</a>
          <a href="{{ site.baseurl }}/guides">Guides</a>
          <a href="{% api_doc_root %}">API</a>
          <a href="https://tinyletter.com/graphql-ruby">Newsletter</a>
          <a href="https://github.com/rmosolgo/graphql-ruby">Source Code</a>
          <a href="https://graphql.pro">Upgrade to Pro</a>
          <input
            class="search-input"
            onkeyup="GraphQLRubySearch.run(this)"
            type="text"
            placeholder="Search the docs..."
          />
        </div>
      </div>
      <div class="search-results-container">
        <div id="search-results">
        </div>
      </div>
    </div>
    <div class="container">
      {{ content }}
    </div>
    <script src="https://cdn.jsdelivr.net/algoliasearch/3/algoliasearchLite.min.js"></script>
    <script src="{{ site.baseurl }}/js/search.js"></script>
  </body>
</html>
