query AbstractFragments {
  node(id: "1") {
    ...Frag1
  }
}

fragment Frag1 on Commentable {
  id
  __typename
  ...Frag2
}

fragment Frag2 on Commentable {
  id
  __typename
  ...Frag3
}

fragment Frag3 on Commentable {
  id
  __typename
  ...Frag4
}

fragment Frag4 on Commentable {
  id
  __typename
  ...Frag5
}

fragment Frag5 on Commentable {
  id
  __typename
  ...Frag6
}

fragment Frag6 on Commentable {
  comments {
    body
  }
}
