# frozen_string_literal: true

<% module_namespacing_when_supported do -%>
module Types
  class <%= ruby_class_name %> < Types::BaseObject
<% if options.node %>    implements GraphQL::Types::Relay::Node
<% end %><% normalized_fields.each do |f| %>    <%= f.to_object_field %>
<% end %>  end
end
<% end -%>
