# frozen_string_literal: true
require "graphql/types/boolean"
require "graphql/types/big_int"
require "graphql/types/float"
require "graphql/types/id"
require "graphql/types/int"
require "graphql/types/iso_8601_date"
require "graphql/types/iso_8601_date_time"
require "graphql/types/json"
require "graphql/types/string"
require "graphql/types/relay"
