# frozen_string_literal: true
require "graphql/internal_representation/document"
require "graphql/internal_representation/node"
require "graphql/internal_representation/print"
require "graphql/internal_representation/rewrite"
require "graphql/internal_representation/scope"
require "graphql/internal_representation/visit"
