# frozen_string_literal: true
require "graphql/compatibility/execution_specification"
require "graphql/compatibility/lazy_execution_specification"
require "graphql/compatibility/query_parser_specification"
require "graphql/compatibility/schema_parser_specification"
